@@ -1116,6 +1116,11 @@ static void CompressClusterFit(const size_t count,
   gridrcp.z = 1.0f/31.0f;
   gridrcp.w = 0.0f;
 
+  xSumwSum.x = 0.0f;
+  xSumwSum.y = 0.0f;
+  xSumwSum.z = 0.0f;
+  xSumwSum.w = 0.0f;
+
   ConstructOrdering(count,points,principle,pointsWeights,&xSumwSum,order,0);
 
   for (iterationIndex = 0;;)
@@ -2572,9 +2577,7 @@ static MagickBooleanType WriteDDSImage(const ImageInfo *image_info,
   status=OpenBlob(image_info,image,WriteBinaryBlobMode,&image->exception);
   if (status == MagickFalse)
     return(status);
-  if (IssRGBCompatibleColorspace(image->colorspace) == MagickFalse)
-    (void) TransformImageColorspace(image,sRGBColorspace);
-
+  (void) TransformImageColorspace(image,sRGBColorspace);
   pixelFormat=DDPF_FOURCC;
   compression=FOURCC_DXT5;
 
@@ -2601,7 +2604,7 @@ static MagickBooleanType WriteDDSImage(const ImageInfo *image_info,
       option=GetImageOption(image_info,"dds:cluster-fit");
       if (option != (char *) NULL && LocaleCompare(option,"true") == 0)
         {
-          clusterFit=MagickFalse;
+          clusterFit=MagickTrue;
           if (compression != FOURCC_DXT1)
             {
               option=GetImageOption(image_info,"dds:weight-by-alpha");
@@ -2649,6 +2652,9 @@ static MagickBooleanType WriteDDSImage(const ImageInfo *image_info,
 static void WriteDDSInfo(Image *image, const size_t pixelFormat,
   const size_t compression, const size_t mipmaps)
 {
+  char
+    software[MaxTextExtent];
+
   register ssize_t
     i;
 
@@ -2686,7 +2692,9 @@ static void WriteDDSInfo(Image *image, const size_t pixelFormat,
 
   (void) WriteBlobLSBLong(image,0x00);
   (void) WriteBlobLSBLong(image,(unsigned int) mipmaps+1);
-  (void) WriteBlob(image,44,(unsigned char *) "IMAGEMAGICK");
+  (void) ResetMagickMemory(software,0,sizeof(software));
+  (void) strcpy(software,"IMAGEMAGICK");
+  (void) WriteBlob(image,44,(unsigned char *) software);
 
   (void) WriteBlobLSBLong(image,32);
   (void) WriteBlobLSBLong(image,format);
@@ -2971,22 +2979,29 @@ static MagickBooleanType WriteMipmaps(Image *image, const size_t pixelFormat,
 static void WriteSingleColorFit(Image *image, const DDSVector4* points,
   const ssize_t* map)
 {
-  unsigned char
-    color[3],
-    index,
-    indices[16];
-
   DDSVector3
     start,
     end;
 
+  register ssize_t
+    i;
+
+  unsigned char
+    color[3],
+    index,
+    indexes[16],
+    indices[16];
+
   color[0] = (unsigned char) ClampToLimit(255.0f*points->x,255);
   color[1] = (unsigned char) ClampToLimit(255.0f*points->y,255);
   color[2] = (unsigned char) ClampToLimit(255.0f*points->z,255);
 
+  index=0;
   ComputeEndPoints(DDS_LOOKUP,color,&start,&end,&index);
 
-  RemapIndices(map,&index,indices);
+  for (i=0; i< 16; i++)
+    indexes[i]=index;
+  RemapIndices(map,indexes,indices);
   WriteIndices(image,start,end,indices);
 }
 
