@@ -340,11 +340,10 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
   opj_stream_set_write_function(jp2_stream,JP2WriteHandler);
   opj_stream_set_seek_function(jp2_stream,JP2SeekHandler);
   opj_stream_set_skip_function(jp2_stream,JP2SkipHandler);
-  opj_stream_set_user_data(jp2_stream,image);
+  opj_stream_set_user_data(jp2_stream,image,NULL);
   opj_stream_set_user_data_length(jp2_stream,GetBlobSize(image));
   if (opj_read_header(jp2_stream,jp2_codec,&jp2_image) == 0)
     {
-      opj_stream_set_user_data(jp2_stream,NULL);
       opj_stream_destroy(jp2_stream);
       opj_destroy_codec(jp2_codec);
       ThrowReaderException(DelegateError,"UnableToDecodeImageFile");
@@ -359,7 +358,6 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
         image->extract_info.y+image->rows);
       if (jp2_status == 0)
         {
-          opj_stream_set_user_data(jp2_stream,NULL);
           opj_stream_destroy(jp2_stream);
           opj_destroy_codec(jp2_codec);
           opj_image_destroy(jp2_image);
@@ -377,19 +375,16 @@ static Image *ReadJP2Image(const ImageInfo *image_info,ExceptionInfo *exception)
     }
   if (jp2_status == 0)
     {
-      opj_stream_set_user_data(jp2_stream,NULL);
       opj_stream_destroy(jp2_stream);
       opj_destroy_codec(jp2_codec);
       opj_image_destroy(jp2_image);
       ThrowReaderException(DelegateError,"UnableToDecodeImageFile");
     }
-  opj_stream_set_user_data(jp2_stream,NULL);
   opj_stream_destroy(jp2_stream);
   for (i=0; i < (ssize_t) jp2_image->numcomps; i++)
   {
     if ((jp2_image->comps[i].dx == 0) || (jp2_image->comps[i].dy == 0))
       {
-        opj_stream_set_user_data(jp2_stream,NULL);
         opj_destroy_codec(jp2_codec);
         opj_image_destroy(jp2_image);
         ThrowReaderException(CoderError,"IrregularChannelGeometryNotSupported")
@@ -1028,7 +1023,7 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   opj_stream_set_write_function(jp2_stream,JP2WriteHandler);
   opj_stream_set_seek_function(jp2_stream,JP2SeekHandler);
   opj_stream_set_skip_function(jp2_stream,JP2SkipHandler);
-  opj_stream_set_user_data(jp2_stream,image);
+  opj_stream_set_user_data(jp2_stream,image,NULL);
   if (jp2_stream == (opj_stream_t *) NULL)
     ThrowWriterException(DelegateError,"UnableToEncodeImageFile");
   jp2_status=opj_start_compress(jp2_codec,jp2_image,jp2_stream);
@@ -1037,7 +1032,6 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   if ((opj_encode(jp2_codec,jp2_stream) == 0) ||
       (opj_end_compress(jp2_codec,jp2_stream) == 0))
     {
-      opj_stream_set_user_data(jp2_stream,NULL);
       opj_stream_destroy(jp2_stream);
       opj_destroy_codec(jp2_codec);
       opj_image_destroy(jp2_image);
@@ -1046,7 +1040,6 @@ static MagickBooleanType WriteJP2Image(const ImageInfo *image_info,Image *image)
   /*
     Free resources.
   */
-  opj_stream_set_user_data(jp2_stream,NULL);
   opj_stream_destroy(jp2_stream);
   opj_destroy_codec(jp2_codec);
   opj_image_destroy(jp2_image);
