@@ -98,6 +98,16 @@
 #define cmsOpenProfileFromMemTHR(context,profile,length) \
   cmsOpenProfileFromMem(profile,length)
 #endif
+
+/*
+  Forward declarations
+*/
+static MagickBooleanType
+  SetImageProfileInternal(Image *,const char *,const StringInfo *,
+    const MagickBooleanType);
+
+static void
+  WriteTo8BimProfile(Image *,const char*,const StringInfo *);
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
@@ -195,6 +205,7 @@ MagickExport MagickBooleanType DeleteImageProfile(Image *image,const char *name)
       image->iptc_profile.length=0;
       image->iptc_profile.info=(unsigned char *) NULL;
     }
+  WriteTo8BimProfile(image,name,(StringInfo *) NULL);
   return(DeleteNodeFromSplayTree((SplayTreeInfo *) image->profiles,name));
 }
 
@@ -1024,6 +1035,7 @@ MagickExport StringInfo *RemoveImageProfile(Image *image,const char *name)
       image->iptc_profile.length=0;
       image->iptc_profile.info=(unsigned char *) NULL;
     }
+  WriteTo8BimProfile(image,name,(StringInfo *) NULL);
   profile=(StringInfo *) RemoveNodeFromSplayTree((SplayTreeInfo *)
     image->profiles,name);
   return(profile);
@@ -1136,7 +1148,127 @@ static inline const unsigned char *ReadResourceShort(const unsigned char *p,
   return(p);
 }
 
-static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
+static inline void WriteResourceLong(unsigned char *p,
+  const unsigned int quantum)
+{
+  unsigned char
+    buffer[4];
+
+  buffer[0]=(unsigned char) (quantum >> 24);
+  buffer[1]=(unsigned char) (quantum >> 16);
+  buffer[2]=(unsigned char) (quantum >> 8);
+  buffer[3]=(unsigned char) quantum;
+  (void) CopyMagickMemory(p,buffer,4);
+}
+
+static void WriteTo8BimProfile(Image *image,const char *name,
+  const StringInfo *profile)
+{
+
+  const unsigned char
+    *datum,
+    *s;
+
+  register const unsigned char
+    *p;
+
+  size_t
+    length;
+
+  StringInfo
+    *profile_8bim;
+
+  ssize_t
+    count;
+
+  unsigned char
+    length_byte;
+
+  unsigned int
+    value;
+
+  unsigned short
+    id,
+    profile_id;
+
+  if (LocaleCompare(name,"icc") == 0)
+    profile_id=0x040f;
+  else if (LocaleCompare(name,"iptc") == 0)
+    profile_id=0x0404;
+  else if (LocaleCompare(name,"xmp") == 0)
+    profile_id=0x0424;
+  else
+    return;
+  profile_8bim=(StringInfo *)GetValueFromSplayTree((SplayTreeInfo *)
+    image->profiles,"8bim");
+  if (profile_8bim == (StringInfo *) NULL)
+    return;
+  datum=GetStringInfoDatum(profile_8bim);
+  length=GetStringInfoLength(profile_8bim);
+  for (p=datum; p < (datum+length-16); )
+  {
+    s=p;
+    if (LocaleNCompare((char *) p,"8BIM",4) != 0)
+      break;
+    p+=4;
+    p=ReadResourceShort(p,&id);
+    p=ReadResourceByte(p,&length_byte);
+    p+=length_byte;
+    if (((length_byte+1) & 0x01) != 0)
+      p++;
+    if (p > (datum+length-4))
+      break;
+    p=ReadResourceLong(p,&value);
+    count=(ssize_t) value;
+    if ((p > (datum+length-count)) || (count > length))
+      break;
+    if ((count & 0x01) != 0)
+      count++;
+    if (id == profile_id)
+      {
+        size_t
+          offset,
+          rest;
+
+        ssize_t
+          new_count;
+
+        StringInfo
+          *new_profile;
+
+        new_count=0;
+        rest=(datum+length)-(p+count);
+        if (profile == (StringInfo *) NULL)
+          {
+            offset=(s-datum);
+            new_profile=AcquireStringInfo(offset+rest);
+            (void) CopyMagickMemory(new_profile->datum,datum,offset);
+          }
+        else
+          {
+            offset=(p-datum);
+            new_count=profile->length;
+            if ((new_count & 0x01) != 0)
+              new_count++;
+            new_profile=AcquireStringInfo(offset+new_count+rest);
+            (void) CopyMagickMemory(new_profile->datum,datum,offset-4);
+            WriteResourceLong(new_profile->datum+offset-4,profile->length);
+            (void) CopyMagickMemory(new_profile->datum+offset,profile->datum,
+              profile->length);
+          }
+        (void) CopyMagickMemory(new_profile->datum+offset+new_count,p+count,
+          rest);
+        (void) AddValueToSplayTree((SplayTreeInfo *) image->profiles,
+          ConstantString("8bim"),CloneStringInfo(new_profile));
+        new_profile=DestroyStringInfo(new_profile);
+        break;
+      }
+    else
+      p+=count;
+  }
+}
+
+static void GetProfilesFromResourceBlock(Image *image,
   const StringInfo *resource_block)
 {
   const unsigned char
@@ -1219,7 +1351,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         */
         profile=AcquireStringInfo(count);
         SetStringInfoDatum(profile,p);
-        (void) SetImageProfile(image,"iptc",profile);
+        (void) SetImageProfileInternal(image,"iptc",profile,MagickTrue);
         profile=DestroyStringInfo(profile);
         p+=count;
         break;
@@ -1239,7 +1371,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         */
         profile=AcquireStringInfo(count);
         SetStringInfoDatum(profile,p);
-        (void) SetImageProfile(image,"icc",profile);
+        (void) SetImageProfileInternal(image,"icc",profile,MagickTrue);
         profile=DestroyStringInfo(profile);
         p+=count;
         break;
@@ -1251,7 +1383,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         */
         profile=AcquireStringInfo(count);
         SetStringInfoDatum(profile,p);
-        (void) SetImageProfile(image,"exif",profile);
+        (void) SetImageProfileInternal(image,"exif",profile,MagickTrue);
         profile=DestroyStringInfo(profile);
         p+=count;
         break;
@@ -1263,7 +1395,7 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
         */
         profile=AcquireStringInfo(count);
         SetStringInfoDatum(profile,p);
-        (void) SetImageProfile(image,"xmp",profile);
+        (void) SetImageProfileInternal(image,"xmp",profile,MagickTrue);
         profile=DestroyStringInfo(profile);
         p+=count;
         break;
@@ -1277,11 +1409,10 @@ static MagickBooleanType GetProfilesFromResourceBlock(Image *image,
     if ((count & 0x01) != 0)
       p++;
   }
-  return(MagickTrue);
 }
 
-MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
-  const StringInfo *profile)
+static MagickBooleanType SetImageProfileInternal(Image *image,const char *name,
+  const StringInfo *profile,const MagickBooleanType recursive)
 {
   char
     key[MaxTextExtent],
@@ -1298,6 +1429,7 @@ MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
     image->profiles=NewSplayTree(CompareSplayTreeString,RelinquishMagickMemory,
       DestroyProfile);
   (void) CopyMagickString(key,name,MaxTextExtent);
+  LocaleLower(key);
   status=AddValueToSplayTree((SplayTreeInfo *) image->profiles,
     ConstantString(key),CloneStringInfo(profile));
   if ((status != MagickFalse) &&
@@ -1331,7 +1463,13 @@ MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
           image->iptc_profile.length=GetStringInfoLength(iptc_profile);
           image->iptc_profile.info=GetStringInfoDatum(iptc_profile);
         }
-      (void) GetProfilesFromResourceBlock(image,profile);
+    }
+  if (status != MagickFalse)
+    {
+      if (LocaleCompare(name,"8bim") == 0)
+        GetProfilesFromResourceBlock(image,profile);
+      else if (recursive == MagickFalse)
+        WriteTo8BimProfile(image,name,profile);
     }
   /*
     Inject profile into image properties.
@@ -1340,6 +1478,12 @@ MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
   (void) GetImageProperty(image,property);
   return(status);
 }
+
+MagickExport MagickBooleanType SetImageProfile(Image *image,const char *name,
+  const StringInfo *profile)
+{
+  return(SetImageProfileInternal(image,name,profile,MagickFalse));
+}
 
 /*
 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
