@@ -305,6 +305,7 @@ MagickExport void RegisterStaticModules(void)
   (void) RegisterVICARImage();
   (void) RegisterVIDImage();
   (void) RegisterVIFFImage();
+  (void) RegisterVIPSImage();
   (void) RegisterWBMPImage();
 #if defined(MAGICKCORE_WEBP_DELEGATE)
   (void) RegisterWEBPImage();
@@ -491,6 +492,7 @@ MagickExport void UnregisterStaticModules(void)
   UnregisterVICARImage();
   UnregisterVIDImage();
   UnregisterVIFFImage();
+  UnregisterVIPSImage();
   UnregisterWBMPImage();
 #if defined(MAGICKCORE_WEBP_DELEGATE)
   UnregisterWEBPImage();
