@@ -171,6 +171,7 @@ extern ModuleExport size_t
   RegisterVICARImage(void),
   RegisterVIDImage(void),
   RegisterVIFFImage(void),
+  RegisterVIPSImage(void),
   RegisterVSTImage(void),
   RegisterWBMPImage(void),
   RegisterWEBPImage(void),
@@ -334,6 +335,7 @@ extern ModuleExport void
   UnregisterVICARImage(void),
   UnregisterVIDImage(void),
   UnregisterVIFFImage(void),
+  UnregisterVIPSImage(void),
   UnregisterVSTImage(void),
   UnregisterWBMPImage(void),
   UnregisterWEBPImage(void),
